"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutomaticCCTPRoute = void 0;
const sdk_base_1 = require("@wormhole-foundation/sdk-base");
const cctpTransfer_1 = require("../../protocols/cctpTransfer");
const types_1 = require("../../types");
const route_1 = require("../route");
class AutomaticCCTPRoute extends route_1.AutomaticRoute {
    NATIVE_GAS_DROPOFF_SUPPORTED = true;
    getDefaultOptions() {
        return {
            nativeGas: 0.0,
        };
    }
    async isSupported() {
        if (!this.request.toChain.supportsCircleBridge() ||
            !this.request.fromChain.supportsCircleBridge()) {
            return false;
        }
        if (!sdk_base_1.circle.usdcContract.get(this.wh.network, this.request.from.chain)) {
            return false;
        }
        if (!sdk_base_1.circle.usdcContract.get(this.wh.network, this.request.to.chain)) {
            return false;
        }
        return true;
    }
    async isAvailable() {
        return false;
    }
    async validate(params) {
        try {
            const options = params.options ?? this.getDefaultOptions();
            const normalizedParams = await this.normalizeTransferParams(params);
            if (normalizedParams.amount <= 0n) {
                return {
                    valid: false,
                    params,
                    error: new Error("Amount must be positive"),
                };
            }
            const validatedParams = {
                normalizedParams,
                options,
                ...params,
            };
            return { valid: true, params: validatedParams };
        }
        catch (e) {
            return {
                valid: false,
                params,
                error: e,
            };
        }
    }
    async quote(params) {
        return await cctpTransfer_1.CircleTransfer.quoteTransfer(this.request.fromChain, this.request.toChain, this.toTransferDetails(params));
    }
    async normalizeTransferParams(params) {
        const amount = this.request.normalizeAmount(params.amount);
        const ctb = await this.request.fromChain.getAutomaticCircleBridge();
        const fee = await ctb.getRelayerFee(this.request.to.chain);
        const minAmount = (fee * 105n) / 100n;
        if (amount < minAmount) {
            throw new Error(`Minimum amount is ${this.request.displayAmount(minAmount)}`);
        }
        const transferableAmount = amount - fee;
        const options = params.options ?? this.getDefaultOptions();
        const nativeGasPerc = options.nativeGas ?? 0.0;
        if (nativeGasPerc > 1.0 || nativeGasPerc < 0.0)
            throw new Error("Native gas must be between 0.0 and 1.0 (0% and 100%)");
        let nativeGasAmount = 0n;
        if (nativeGasPerc > 0.0) {
            const scale = 10000;
            const scaledGas = BigInt(nativeGasPerc * scale);
            nativeGasAmount = (transferableAmount * scaledGas) / BigInt(scale);
        }
        return { fee, amount, nativeGasAmount };
    }
    toTransferDetails(params) {
        return {
            from: this.request.from,
            to: this.request.to,
            amount: params.normalizedParams.amount,
            automatic: true,
            nativeGas: params.normalizedParams.nativeGasAmount,
        };
    }
    async initiate(signer, params) {
        let transfer = this.toTransferDetails(params);
        let txids = await cctpTransfer_1.CircleTransfer.transfer(this.request.fromChain, transfer, signer);
        // const msg = await CircleTransfer.getTransferMessage(
        //   this.request.fromChain,
        //   txids[txids.length - 1]!.txid,
        // );
        const [msgid] = await this.request.fromChain.parseTransaction(txids[txids.length - 1].txid);
        return {
            from: transfer.from.chain,
            to: transfer.to.chain,
            state: types_1.TransferState.SourceFinalized,
            originTxs: txids,
            attestation: { id: msgid },
        };
    }
    async *track(receipt, timeout) {
        yield* cctpTransfer_1.CircleTransfer.track(this.wh, receipt, timeout, this.request.fromChain, this.request.toChain);
    }
}
exports.AutomaticCCTPRoute = AutomaticCCTPRoute;
//# sourceMappingURL=automatic.js.map