"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CCTPRoute = void 0;
const sdk_base_1 = require("@wormhole-foundation/sdk-base");
const sdk_definitions_1 = require("@wormhole-foundation/sdk-definitions");
const common_1 = require("../../common");
const cctpTransfer_1 = require("../../protocols/cctpTransfer");
const types_1 = require("../../types");
const route_1 = require("../route");
class CCTPRoute extends route_1.ManualRoute {
    getDefaultOptions() {
        return {
            payload: undefined,
        };
    }
    async isSupported() {
        if (!this.request.toChain.supportsCircleBridge() ||
            !this.request.fromChain.supportsCircleBridge()) {
            return false;
        }
        if (!sdk_base_1.circle.usdcContract.get(this.wh.network, this.request.from.chain)) {
            return false;
        }
        if (!sdk_base_1.circle.usdcContract.get(this.wh.network, this.request.to.chain)) {
            return false;
        }
        return true;
    }
    async validate(params) {
        const amount = this.request.normalizeAmount(params.amount);
        if (amount < 0n) {
            return {
                valid: false,
                params,
                error: new Error("Amount must be positive"),
            };
        }
        const validatedParams = {
            normalizedParams: {
                amount,
            },
            options: params.options ?? this.getDefaultOptions(),
            ...params,
        };
        return { valid: true, params: validatedParams };
    }
    async quote(params) {
        return await cctpTransfer_1.CircleTransfer.quoteTransfer(this.request.fromChain, this.request.toChain, this.toTransferDetails(params));
    }
    async initiate(signer, params) {
        let transfer = this.toTransferDetails(params);
        let txids = await cctpTransfer_1.CircleTransfer.transfer(this.request.fromChain, transfer, signer);
        const msg = await cctpTransfer_1.CircleTransfer.getTransferMessage(this.request.fromChain, txids[txids.length - 1].txid);
        return {
            from: transfer.from.chain,
            to: transfer.to.chain,
            state: types_1.TransferState.SourceFinalized,
            originTxs: txids,
            attestation: { id: msg },
        };
    }
    async complete(signer, receipt) {
        if (!(0, types_1.isAttested)(receipt))
            throw new Error("The source must be finalized in order to complete the transfer");
        const { id, attestation: att } = receipt.attestation;
        if (sdk_definitions_1.CircleBridge.isCircleAttestation(att)) {
            const { message, attestation } = att;
            if (!attestation)
                throw new Error(`No Circle attestation for ${id}`);
            let cb = await this.request.toChain.getCircleBridge();
            let xfer = cb.redeem(this.request.to.address, message, attestation);
            return await (0, common_1.signSendWait)(this.request.toChain, xfer, signer);
        }
        else {
            //
            return [];
        }
    }
    async *track(receipt, timeout) {
        yield* cctpTransfer_1.CircleTransfer.track(this.wh, receipt, timeout, this.request.fromChain, this.request.toChain);
    }
    toTransferDetails(params) {
        return {
            from: this.request.from,
            to: this.request.to,
            amount: params.normalizedParams.amount,
            automatic: false,
            ...params.options,
        };
    }
}
exports.CCTPRoute = CCTPRoute;
//# sourceMappingURL=manual.js.map