"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenBridgeRoute = void 0;
const sdk_definitions_1 = require("@wormhole-foundation/sdk-definitions");
const tokenTransfer_1 = require("../../protocols/tokenTransfer");
const types_1 = require("../../types");
const route_1 = require("../route");
class TokenBridgeRoute extends route_1.ManualRoute {
    getDefaultOptions() {
        return { payload: undefined };
    }
    async isSupported() {
        try {
            // No transfers to same chain
            if (this.request.fromChain.chain === this.request.toChain.chain)
                return false;
            // No transfers to unsupported chains
            if (!this.request.fromChain.supportsTokenBridge())
                return false;
            if (!this.request.toChain.supportsTokenBridge())
                return false;
            // Ensure the destination token is the equivalent wrapped token
            let { source, destination } = this.request;
            if (destination && (0, sdk_definitions_1.isTokenId)(destination.id)) {
                let equivalentToken = await tokenTransfer_1.TokenTransfer.lookupDestinationToken(this.request.fromChain, this.request.toChain, source.id);
                if (!(0, sdk_definitions_1.isSameToken)(equivalentToken, destination.id)) {
                    return false;
                }
            }
            else if (destination && destination.id === "native") {
                return false;
            }
        }
        catch (e) {
            return false;
        }
        return true;
    }
    async validate(params) {
        const amt = this.request.normalizeAmount(params.amount);
        if (amt <= 0n) {
            return { valid: false, params, error: new Error("Amount has to be positive") };
        }
        const validatedParams = {
            amount: params.amount,
            normalizedParams: { amount: amt },
            options: {},
        };
        return { valid: true, params: validatedParams };
    }
    async quote(params) {
        return await tokenTransfer_1.TokenTransfer.quoteTransfer(this.request.fromChain, this.request.toChain, this.toTransferDetails(params));
    }
    async initiate(signer, params) {
        const transfer = this.toTransferDetails(params);
        const txids = await tokenTransfer_1.TokenTransfer.transfer(this.request.fromChain, transfer, signer);
        const msg = await tokenTransfer_1.TokenTransfer.getTransferMessage(this.request.fromChain, txids[txids.length - 1].txid);
        return {
            from: transfer.from.chain,
            to: transfer.to.chain,
            state: types_1.TransferState.SourceFinalized,
            originTxs: txids,
            attestation: { id: msg },
        };
    }
    async complete(signer, receipt) {
        if (!(0, types_1.isAttested)(receipt))
            throw new Error("The source must be finalized in order to complete the transfer");
        return await tokenTransfer_1.TokenTransfer.redeem(this.request.toChain, 
        // todo: ew?
        receipt.attestation.attestation, signer);
    }
    async *track(receipt, timeout) {
        yield* tokenTransfer_1.TokenTransfer.track(this.wh, receipt, timeout, this.request.fromChain, this.request.toChain);
    }
    toTransferDetails(params) {
        return {
            token: this.request.source.id,
            from: this.request.from,
            to: this.request.to,
            amount: params.normalizedParams.amount,
            ...params.options,
        };
    }
}
exports.TokenBridgeRoute = TokenBridgeRoute;
//# sourceMappingURL=manual.js.map